"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvmAddress = exports.EvmZeroAddress = void 0;
const sdk_connect_1 = require("@wormhole-foundation/sdk-connect");
const ethers_1 = require("ethers");
const types_js_1 = require("./types.js");
exports.EvmZeroAddress = '0x0000000000000000000000000000000000000000';
class EvmAddress {
    static byteSize = 20;
    static platform = types_js_1._platform;
    type = 'Native';
    // stored as checksum address
    address;
    constructor(address) {
        if (EvmAddress.instanceof(address)) {
            const a = address;
            this.address = a.address;
            return;
        }
        if (typeof address === 'string') {
            if (!EvmAddress.isValidAddress(address))
                throw new Error(`Invalid EVM address, expected ${EvmAddress.byteSize}-byte hex string but got ${address}`);
            this.address = (0, ethers_1.getAddress)(address);
        }
        else if (address instanceof Uint8Array) {
            if (address.length !== EvmAddress.byteSize)
                throw new Error(`Invalid EVM address, expected ${EvmAddress.byteSize} bytes but got ${address.length}`);
            this.address = (0, ethers_1.getAddress)(sdk_connect_1.encoding.hex.encode(address));
        }
        else if (sdk_connect_1.UniversalAddress.instanceof(address)) {
            // If its a universal address and we want it to be an ethereum address,
            // we need to chop off the first 12 bytes of padding
            const addressBytes = address.toUint8Array();
            // double check to make sure there are no non zero bytes
            if (addressBytes.slice(0, 12).some((v) => {
                v !== 0;
            }))
                throw new Error(`Invalid EVM address ${address}`);
            const suffix = sdk_connect_1.encoding.hex.encode(addressBytes.slice(12));
            this.address = (0, ethers_1.getAddress)(suffix);
        }
        else
            throw new Error(`Invalid EVM address ${address}`);
    }
    unwrap() {
        return this.address;
    }
    toString() {
        return this.address;
    }
    toNative() {
        return this;
    }
    toUint8Array() {
        return sdk_connect_1.encoding.hex.decode(this.address);
    }
    toUniversalAddress() {
        return new sdk_connect_1.UniversalAddress(this.address, 'hex');
    }
    static isValidAddress(address) {
        return (0, ethers_1.isAddress)(address);
    }
    static instanceof(address) {
        return address.constructor.platform === EvmAddress.platform;
    }
    equals(other) {
        if (EvmAddress.instanceof(other)) {
            return other.address === this.address;
        }
        else {
            return other.equals(this.toUniversalAddress());
        }
    }
}
exports.EvmAddress = EvmAddress;
(0, sdk_connect_1.registerNative)(types_js_1._platform, EvmAddress);
//# sourceMappingURL=address.js.map